@@ -11,7 +11,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: buffer.c,v 1.22 2003/09/01 20:15:19 volsung Exp $
+ last mod: $Id: buffer.c 16870 2010-02-04 14:15:42Z xiphmont $
 
  ********************************************************************/
 
@@ -114,7 +114,7 @@ action_t *malloc_action (action_func_t action_func, void *action_arg)
   action = malloc(sizeof(action_t));
 
   if (action == NULL) {
-    fprintf(stderr, _("Error: Out of memory in malloc_action().\n"));
+    fprintf(stderr, _("ERROR: Out of memory in malloc_action().\n"));
     exit(1);
   }
 
@@ -207,7 +207,7 @@ void *buffer_thread_func (void *arg)
 
   /* This test is safe since curfill will never decrease and eos will
      never be unset. */
-  while ( !(buf->eos && buf->curfill == 0)) {
+  while ( !(buf->eos && buf->curfill == 0) && !buf->abort_write) {
 
     if (buf->cancel_flag || sig_request.cancel)
       break;
@@ -243,25 +243,34 @@ void *buffer_thread_func (void *arg)
 
     UNLOCK_MUTEX(buf->mutex);
  
-    /* No need to lock mutex here because the other thread will
-       NEVER reduce the number of bytes stored in the buffer */
-    DEBUG1("Sending %d bytes to the audio device", write_amount);
-    write_amount = buf->write_func(buf->buffer + buf->start, write_amount,
-		    /* Only set EOS if this is the last chunk */
-		    write_amount == buf->curfill ? buf->eos : 0,
-		    buf->write_arg);
+    if(write_amount){ /* we might have been woken spuriously */
+      /* No need to lock mutex here because the other thread will
+         NEVER reduce the number of bytes stored in the buffer */
+      DEBUG1("Sending %d bytes to the audio device", write_amount);
+      write_amount = buf->write_func(buf->buffer + buf->start, write_amount,
+                                     /* Only set EOS if this is the last chunk */
+                                     write_amount == buf->curfill ? buf->eos : 0,
+                                     buf->write_arg);
 
-    LOCK_MUTEX(buf->mutex);
+      if (!write_amount) {
+        DEBUG("Error writing to the audio device. Aborting.");
+        buffer_abort_write(buf);
+      }
 
-    buf->curfill -= write_amount;
-    buf->position += write_amount;
-    buf->start = (buf->start + write_amount) % buf->size;
-    DEBUG1("Updated buffer fill, curfill = %ld", buf->curfill);
+      LOCK_MUTEX(buf->mutex);
 
-    /* If we've essentially emptied the buffer and prebuffering is enabled,
-       we need to do another prebuffering session */
-    if (!buf->eos && (buf->curfill < buf->audio_chunk_size))
-      buf->prebuffering = buf->prebuffer_size > 0;
+      buf->curfill -= write_amount;
+      buf->position += write_amount;
+      buf->start = (buf->start + write_amount) % buf->size;
+      DEBUG1("Updated buffer fill, curfill = %ld", buf->curfill);
+
+      /* If we've essentially emptied the buffer and prebuffering is enabled,
+         we need to do another prebuffering session */
+      if (!buf->eos && (buf->curfill < buf->audio_chunk_size))
+        buf->prebuffering = buf->prebuffer_size > 0;
+    }else{
+      DEBUG("Woken spuriously");
+    }
 
     /* Signal a waiting decoder thread that they can put more audio into the
        buffer */
@@ -278,7 +287,7 @@ void *buffer_thread_func (void *arg)
 }
 
 
-void submit_data_chunk (buf_t *buf, char *data, size_t size)
+int submit_data_chunk (buf_t *buf, char *data, size_t size)
 {
   long   buf_write_pos; /* offset of first available write location */
   size_t write_size;
@@ -331,7 +340,7 @@ void submit_data_chunk (buf_t *buf, char *data, size_t size)
     if (!buf->prebuffering && !buf->paused) {
 
       DEBUG("Signalling playback thread that more data is available.");
-      COND_SIGNAL(buf->playback_cond);      
+      COND_SIGNAL(buf->playback_cond);
     } else
       DEBUG("Not signalling playback thread since prebuffering or paused.");
 
@@ -341,6 +350,7 @@ void submit_data_chunk (buf_t *buf, char *data, size_t size)
   pthread_cleanup_pop(0);
 
   DEBUG("Exit submit_data_chunk");
+  return !buf->abort_write;
 }
 
 
@@ -351,7 +361,7 @@ buffer_stats_t *malloc_buffer_stats ()
   new_stats = malloc(sizeof(buffer_stats_t));
 
   if (new_stats == NULL) {
-    fprintf(stderr, _("Error: Could not allocate memory in malloc_buffer_stats()\n"));
+    fprintf(stderr, _("ERROR: Could not allocate memory in malloc_buffer_stats()\n"));
     exit(1);
   }
 
@@ -515,8 +525,8 @@ void buffer_thread_kill (buf_t *buf)
 
 /* --- Data buffering functions --- */
 
-void buffer_submit_data (buf_t *buf, char *data, long nbytes) {
-  submit_data_chunk (buf, data, nbytes);
+int buffer_submit_data (buf_t *buf, char *data, long nbytes) {
+  return submit_data_chunk (buf, data, nbytes);
 }
 
 size_t buffer_get_data (buf_t *buf, char *data, long nbytes)
@@ -630,7 +640,7 @@ void buffer_action_now (buf_t *buf, action_func_t action_func,
 			void *action_arg)
 {
   action_t *action;
-  
+
   action = malloc_action(action_func, action_arg);
 
   pthread_cleanup_push(buffer_mutex_unlock, buf);
@@ -746,7 +756,7 @@ void buffer_wait_for_empty (buf_t *buf)
   pthread_cleanup_push(buffer_mutex_unlock, buf);
 
   LOCK_MUTEX(buf->mutex);
-  while (!empty) {
+  while (!empty && !buf->abort_write) {
 
     if (buf->curfill > 0) {
       DEBUG1("Buffer curfill = %ld, going back to sleep.", buf->curfill);
