@@ -11,10 +11,14 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: callbacks.c,v 1.9 2003/09/05 19:25:30 volsung Exp $
+ last mod: $Id: callbacks.c 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <string.h>
 
@@ -43,19 +47,20 @@ void audio_reopen_action (buf_t *buf, void *arg)
   ao_sample_format format;
 
   close_audio_devices (reopen_arg->devices);
-  
+
   /* Record audio device settings and open the devices */
   format.rate = reopen_arg->format->rate;
   format.channels = reopen_arg->format->channels;
   format.bits = reopen_arg->format->word_size * 8;
   format.byte_format = reopen_arg->format->big_endian ? 
     AO_FMT_BIG : AO_FMT_LITTLE;
+  format.matrix = reopen_arg->format->matrix;
 
   current = reopen_arg->devices;
 
   while (current != NULL) {
     ao_info *info = ao_driver_info(current->driver_id);
-        
+
     if (current->filename == NULL)
       current->device = ao_open_live(current->driver_id, &format,
 				     current->options);
@@ -63,50 +68,52 @@ void audio_reopen_action (buf_t *buf, void *arg)
       current->device = ao_open_file(current->driver_id, current->filename,
 				     1 /*overwrite*/, &format, 
 				     current->options);
-    
+
     /* Report errors */
     if (current->device == NULL) {
       switch (errno) {
       case AO_ENODRIVER:
-        status_error(_("Error: Device not available.\n"));
+        status_error(_("ERROR: Device not available.\n"));
 	break;
       case AO_ENOTLIVE:
-	status_error(_("Error: %s requires an output filename to be specified with -f.\n"), info->short_name);
+	status_error(_("ERROR: %s requires an output filename to be specified with -f.\n"), info->short_name);
 	break;
       case AO_EBADOPTION:
-	status_error(_("Error: Unsupported option value to %s device.\n"),
+	status_error(_("ERROR: Unsupported option value to %s device.\n"),
 		     info->short_name);
 	break;
       case AO_EOPENDEVICE:
-	status_error(_("Error: Cannot open device %s.\n"),
+	status_error(_("ERROR: Cannot open device %s.\n"),
 		     info->short_name);
 	break;
       case AO_EFAIL:
-	status_error(_("Error: Device %s failure.\n"), info->short_name);
+	status_error(_("ERROR: Device %s failure.\n"), info->short_name);
 	break;
       case AO_ENOTFILE:
-	status_error(_("Error: An output file cannot be given for %s device.\n"), info->short_name);
+	status_error(_("ERROR: An output file cannot be given for %s device.\n"), info->short_name);
 	break;
       case AO_EOPENFILE:
-	status_error(_("Error: Cannot open file %s for writing.\n"),
+	status_error(_("ERROR: Cannot open file %s for writing.\n"),
 		     current->filename);
 	break;
       case AO_EFILEEXISTS:
-	status_error(_("Error: File %s already exists.\n"), current->filename);
+	status_error(_("ERROR: File %s already exists.\n"), current->filename);
 	break;
       default:
-	status_error(_("Error: This error should never happen (%d).  Panic!\n"), errno);
+	status_error(_("ERROR: This error should never happen (%d).  Panic!\n"), errno);
 	break;
       }
 	 
       /* We cannot recover from any of these errors */
-      exit(1);      
+      exit(1);
     }
-    
+
     current = current->next_device;
   }
 
   /* Cleanup argument */
+  if(reopen_arg->format->matrix)
+    free(reopen_arg->format->matrix);
   free(reopen_arg->format);
   free(reopen_arg);
 }
@@ -117,19 +124,21 @@ audio_reopen_arg_t *new_audio_reopen_arg (audio_device_t *devices,
 {
   audio_reopen_arg_t *arg;
 
-  if ( (arg = malloc(sizeof(audio_reopen_arg_t))) == NULL ) {
-    status_error(_("Error: Out of memory in new_audio_reopen_arg().\n"));
+  if ( (arg = calloc(1,sizeof(audio_reopen_arg_t))) == NULL ) {
+    status_error(_("ERROR: Out of memory in new_audio_reopen_arg().\n"));
     exit(1);
-  }  
-  
-  if ( (arg->format = malloc(sizeof(audio_format_t))) == NULL ) {
-    status_error(_("Error: Out of memory in new_audio_reopen_arg().\n"));
+  }
+
+  if ( (arg->format = calloc(1,sizeof(audio_format_t))) == NULL ) {
+    status_error(_("ERROR: Out of memory in new_audio_reopen_arg().\n"));
     exit(1);
-  }  
-  
+  }
+
   arg->devices = devices;
   /* Copy format in case fmt is recycled later */
   *arg->format = *fmt;
+  if(fmt->matrix)
+    arg->format->matrix = strdup(fmt->matrix);
 
   return arg;
 }
@@ -166,11 +175,11 @@ print_statistics_arg_t *new_print_statistics_arg (
 {
   print_statistics_arg_t *arg;
 
-  if ( (arg = malloc(sizeof(print_statistics_arg_t))) == NULL ) {
+  if ( (arg = calloc(1,sizeof(print_statistics_arg_t))) == NULL ) {
     status_error(_("Error: Out of memory in new_print_statistics_arg().\n"));
     exit(1);
-  }  
-  
+  }
+
   arg->stat_format = stat_format;
   arg->data_source_statistics = data_source_statistics;
   arg->decoder_statistics = decoder_statistics;
@@ -225,16 +234,16 @@ status_message_arg_t *new_status_message_arg (int verbosity)
 {
   status_message_arg_t *arg;
 
-  if ( (arg = malloc(sizeof(status_message_arg_t))) == NULL ) {
-    status_error(_("Error: Out of memory in new_status_message_arg().\n"));
+  if ( (arg = calloc(1,sizeof(status_message_arg_t))) == NULL ) {
+    status_error(_("ERROR: Out of memory in new_status_message_arg().\n"));
     exit(1);
-  }  
-  
+  }
+
   arg->verbosity = verbosity;
 
   return arg;
 }
-  
+
 
 void status_error_action (buf_t *buf, void *arg)
 {
@@ -271,17 +280,17 @@ void decoder_buffered_error_callback (void *arg, int severity,
   /* Preformat the string and allocate space for it.  This code taken
      straight from the vsnprintf() man page.  We do this here because
      we might need to reinit ap several times. */
-  if ((sm_arg->message = malloc (size)) == NULL) {
+  if ((sm_arg->message = calloc (size,1)) == NULL) {
     status_error(_("Error: Out of memory in decoder_buffered_metadata_callback().\n"));
     exit(1);
   }
-  
+
   while (1) {
     /* Try to print in the allocated space. */
     va_start(ap, message);
     n = vsnprintf (sm_arg->message, size, message, ap);
     va_end(ap);
-    
+
     /* If that worked, return the string. */
     if (n > -1 && n < size)
       break;
@@ -296,10 +305,10 @@ void decoder_buffered_error_callback (void *arg, int severity,
     }
   }
 
-  
+
   switch (severity) {
   case ERROR:
-    buffer_append_action_at_end(buf, &status_error_action, sm_arg);    
+    buffer_append_action_at_end(buf, &status_error_action, sm_arg);
     break;
   case WARNING:
     sm_arg->verbosity = WARNING_VERBOSITY;
@@ -327,17 +336,17 @@ void decoder_buffered_metadata_callback (void *arg, int verbosity,
   /* Preformat the string and allocate space for it.  This code taken
      straight from the vsnprintf() man page.  We do this here because
      we might need to reinit ap several times. */
-  if ((sm_arg->message = malloc (size)) == NULL) {
-    status_error(_("Error: Out of memory in decoder_buffered_metadata_callback().\n"));
+  if ((sm_arg->message = calloc (size,1)) == NULL) {
+    status_error(_("ERROR: Out of memory in decoder_buffered_metadata_callback().\n"));
     exit(1);
   }
-  
+
   while (1) {
     /* Try to print in the allocated space. */
     va_start(ap, message);
     n = vsnprintf (sm_arg->message, size, message, ap);
     va_end(ap);
-    
+
     /* If that worked, return the string. */
     if (n > -1 && n < size)
       break;
@@ -347,7 +356,7 @@ void decoder_buffered_metadata_callback (void *arg, int verbosity,
     else           /* glibc 2.0 */
       size *= 2;  /* twice the old size */
     if ((sm_arg->message = realloc (sm_arg->message, size)) == NULL) {
-      status_error(_("Error: Out of memory in decoder_buffered_metadata_callback().\n"));
+      status_error(_("ERROR: Out of memory in decoder_buffered_metadata_callback().\n"));
       exit(1);
     }
   }
