@@ -11,7 +11,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: cfgfile_options.c,v 1.3 2002/01/26 11:06:37 segher Exp $
+ last mod: $Id: cfgfile_options.c 17015 2010-03-24 08:21:29Z xiphmont $
 
  ********************************************************************/
 
@@ -20,10 +20,14 @@
  * function for your platform / compiler.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <limits.h> /* for LONG_MAX / LONG_MIN */
+#include <limits.h> /* for INT/LONG_MIN/MAX */
 #include <errno.h>
 
 #include "cfgfile_options.h"
@@ -84,7 +88,7 @@ void file_options_init (file_option_t opts[])
 
       case opt_type_bool:
       case opt_type_int:
-	*(long int *) opts->ptr = *(int *) opts->dfl;
+	*(int *) opts->ptr = *(int *) opts->dfl;
 	break;
 	
       case opt_type_float:
@@ -128,18 +132,18 @@ void file_options_describe (file_option_t opts[], FILE *f)
   /* Column headers */
   /* Name */
   totalWidth += fprintf (f, "%-*s", colWidths[0], _("Name"));
-  
+
   /* Description */
   totalWidth += fprintf (f, "%-*s", colWidths[1], _("Description"));
-  
+
   /* Type */
   totalWidth += fprintf (f, "%-*s", colWidths[2], _("Type"));
-  
+
   /* Default */
   totalWidth += fprintf (f, "%-*s", colWidths[3], _("Default"));
 
   fputc ('\n', f);
-  
+
   /* Divider */
   print_space (f, totalWidth, '-');
 
@@ -203,7 +207,7 @@ void file_options_describe (file_option_t opts[], FILE *f)
 	  break;
 	case opt_type_bool:
 	case opt_type_int:
-	  fprintf (f, "%ld", *(long int *) opt->dfl);
+	  fprintf (f, "%d", *(int *) opt->dfl);
 	  break;
 	case opt_type_float:
 	  fprintf (f, "%f", (double) (*(float *) opt->dfl));
@@ -252,12 +256,12 @@ parse_code_t parse_line (file_option_t opts[], char *line)
   /* remove this if you want a zero-length key */
   if (strlen(line) == 0)
     return parse_nokey;
-  
+
   if (value) {
     /* cut leading whitespace from value */
     while (*value == ' ')
       value++;
-    
+
     /* cut trailing whitespace from value */
     len = strlen (value);
     while (len > 0 && value[len-1] == ' ') {
@@ -288,14 +292,14 @@ parse_code_t parse_line (file_option_t opts[], char *line)
 	/* Maybe this is a numeric bool */
 	tmpl = strtol (value, &endptr, 0);
 
-	if ( strncasecmp(value, "y", 1) == 0
-	     || strcasecmp(value, "true")
+	if ( !strncasecmp(value, "y", 1)
+	     || !strcasecmp(value, "true")
 	     || (*endptr == '\0' && tmpl) )
-	  *(long int *) opt->ptr = 1;
-	else if ( strncasecmp(value, "n", 1) == 0
-		  || strcasecmp(value, "false")
+	  *(int *) opt->ptr = 1;
+	else if ( !strncasecmp(value, "n", 1)
+		  || !strcasecmp(value, "false")
 		  || (*endptr == '\0' && !tmpl) )
-	  *(long int *) opt->ptr = 0;
+	  *(int *) opt->ptr = 0;
 	else
 	  return parse_badvalue;
 	break;
@@ -316,12 +320,15 @@ parse_code_t parse_line (file_option_t opts[], char *line)
       case opt_type_int:
 	if (!value || *value == '\0')
 	  return parse_badvalue;
+	errno = 0;
 	tmpl = strtol (value, &endptr, 0);
 	if (((tmpl == LONG_MIN || tmpl == LONG_MAX) && errno == ERANGE)
 	    || (*endptr != '\0'))
 	  return parse_badvalue;
+	if ((tmpl > INT_MAX) || (tmpl < INT_MIN))
+	  return parse_badvalue;
 	opt->found++;
-	*(long int *) opt->ptr = tmpl;
+	*(int *) opt->ptr = tmpl;
 	break;
 	
       case opt_type_float:
@@ -441,7 +448,7 @@ void parse_std_configs (file_option_t opts[])
   char filename[FILENAME_MAX];
   char *homedir = getenv("HOME");
 
-  parse_config_file(opts, "/etc/ogg123rc");
+  parse_config_file(opts, SYSCONFDIR "/ogg123rc");
   if (homedir && strlen(homedir) < FILENAME_MAX - 10) {
     /* Try ~/.ogg123 */
     strncpy(filename, homedir, FILENAME_MAX);
