@@ -11,7 +11,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: cfgfile_options.h,v 1.2 2001/12/19 02:52:53 volsung Exp $
+ last mod: $Id: cfgfile_options.h 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
@@ -25,7 +25,7 @@ typedef enum {
   opt_type_bool,
   opt_type_char,
   opt_type_string,
-  opt_type_int, /* long int */
+  opt_type_int,
   opt_type_float,
   opt_type_double
 } file_option_type_t;
