@@ -11,10 +11,13 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: cmdline_options.c,v 1.15 2003/09/01 23:54:01 volsung Exp $
+ last mod: $Id: cmdline_options.c 17009 2010-03-24 04:29:17Z xiphmont $
 
  ********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -27,6 +30,9 @@
 
 #define MIN_INPUT_BUFFER_SIZE 8
 
+extern double strtotime(char *s);
+extern void set_seek_opt(ogg123_options_t *ogg123_opts, char *buf);
+
 struct option long_options[] = {
   /* GNU standard options */
     {"help", no_argument, 0, 'h'},
@@ -42,27 +48,18 @@ struct option long_options[] = {
     {"device-option", required_argument, 0, 'o'},
     {"prebuffer", required_argument, 0, 'p'},
     {"quiet", no_argument, 0, 'q'},
+    {"remote", no_argument, 0, 'R'},
     {"verbose", no_argument, 0, 'v'},
     {"nth", required_argument, 0, 'x'},
     {"ntimes", required_argument, 0, 'y'},
     {"shuffle", no_argument, 0, 'z'},
+    {"random", no_argument, 0, 'Z'},
     {"list", required_argument, 0, '@'},
     {"audio-buffer", required_argument, 0, 0},
+    {"repeat", no_argument, 0, 'r'},
     {0, 0, 0, 0}
 };
 
-double strtotime(char *s)
-{
-	double time;
-
-	time = strtod(s, &s);
-
-	while (*s == ':')
-		time = 60 * time + strtod(s + 1, &s);
-
-	return time;
-}
-
 int parse_cmdline_options (int argc, char **argv,
 			   ogg123_options_t *ogg123_opts,
 			   file_option_t    *file_opts)
@@ -75,7 +72,7 @@ int parse_cmdline_options (int argc, char **argv,
   audio_device_t *current;
   int ret;
 
-  while (-1 != (ret = getopt_long(argc, argv, "b:c::d:f:hl:k:K:o:p:qvVx:y:z@:",
+  while (-1 != (ret = getopt_long(argc, argv, "b:c::d:f:hl:k:K:o:p:qrRvVx:y:zZ@:",
 				  long_options, &option_index))) {
 
       switch (ret) {
@@ -149,7 +146,7 @@ int parse_cmdline_options (int argc, char **argv,
 	break;
 
 	case 'k':
-	  ogg123_opts->seekpos = strtotime(optarg);
+ 	  set_seek_opt(ogg123_opts, optarg);
 	  break;
 	  
 	case 'K':
@@ -171,7 +168,7 @@ int parse_cmdline_options (int argc, char **argv,
 	  cmdline_usage();
 	  exit(0);
 	  break;
-	  
+
 	case 'p':
 	  ogg123_opts->input_prebuffer = atof (optarg);
 	  if (ogg123_opts->input_prebuffer < 0.0f || 
@@ -187,12 +184,21 @@ int parse_cmdline_options (int argc, char **argv,
 	ogg123_opts->verbosity = 0;
 	break;
 	
+      case 'r':
+        ogg123_opts->repeat = 1;
+	break;
+
+      case 'R':
+	ogg123_opts->remote = 1;
+	ogg123_opts->verbosity = 0;
+	break;
+
       case 'v':
 	ogg123_opts->verbosity++;
 	break;
 	
       case 'V':
-	status_error(_("ogg123 from %s %s\n"), PACKAGE, VERSION);
+	status_error(_("ogg123 from %s %s"), PACKAGE, VERSION);
 	exit(0);
 	break;
 
@@ -217,6 +223,10 @@ int parse_cmdline_options (int argc, char **argv,
 	ogg123_opts->shuffle = 1;
 	break;
 
+      case 'Z':
+        ogg123_opts->repeat = ogg123_opts->shuffle = 1;
+	break;
+
       case '@':
 	if (playlist_append_from_file(ogg123_opts->playlist, optarg) == 0)
 	  status_error(_("--- Cannot open playlist file %s.  Skipped.\n"),
@@ -234,7 +244,7 @@ int parse_cmdline_options (int argc, char **argv,
 
   /* Sanity check bad option combinations */
   if (ogg123_opts->endpos > 0.0 &&
-      ogg123_opts->seekpos > ogg123_opts->endpos) {
+      ogg123_opts->seekoff > ogg123_opts->endpos) {
     status_error(_("=== Option conflict: End time is before start time.\n"));
     exit(1);
   }
@@ -251,7 +261,7 @@ int parse_cmdline_options (int argc, char **argv,
 	    status_error(_("--- Driver %s specified in configuration file invalid.\n"),
 			 ogg123_opts->default_device);
       }
-      
+
       /* Then try libao autodetect */
       if (temp_driver_id < 0)
 	temp_driver_id = ao_default_driver_id();
@@ -268,55 +278,110 @@ int parse_cmdline_options (int argc, char **argv,
 					     NULL);
     }
 
+  /* if verbosity has been altered, add options to drivers... */
+  {
+    audio_device_t *head = ogg123_opts->devices;
+    while (head){
+      if(ogg123_opts->verbosity>3)
+        ao_append_global_option("debug",NULL);
+      if(ogg123_opts->verbosity>2)
+        ao_append_option(&(head->options),"verbose",NULL);
+      if(ogg123_opts->verbosity==0)
+        ao_append_option(&(head->options),"quiet",NULL);
+      head = head->next_device;
+    }
+  }
+
 
   return optind;
 }
 
+#define LIST_SEP(x) ((x)==0?' ':',')
 
 void cmdline_usage (void)
 {
-  int i, driver_count;
+  int i, j, driver_count;
   ao_info **devices = ao_driver_info_list(&driver_count);
 
-  printf ( 
-         _("ogg123 from %s %s\n"
-	 " by the Xiph.org Foundation (http://www.xiph.org/)\n\n"
-	 "Usage: ogg123 [<options>] <input file> ...\n\n"
-	 "  -h, --help     this help\n"
-	 "  -V, --version  display Ogg123 version\n"
-	 "  -d, --device=d uses 'd' as an output device\n"
-	 "      Possible devices are ('*'=live, '@'=file):\n"
-	 "        "), PACKAGE, VERSION);
-  
-  for(i = 0; i < driver_count; i++) {
-    printf ("%s", devices[i]->short_name);
-    if (devices[i]->type == AO_TYPE_LIVE)
-      printf ("*");
-    else if (devices[i]->type == AO_TYPE_FILE)
-      printf ("@");
-    printf (" ");
-  }
+  printf (_("ogg123 from %s %s\n"
+         " by the Xiph.Org Foundation (http://www.xiph.org/)\n\n"), PACKAGE, VERSION);
 
+  printf (_("Usage: ogg123 [options] file ...\n"
+	    "Play Ogg audio files and network streams.\n\n"));
+ 
+  printf (_("Available codecs: "));
+
+#ifdef HAVE_LIBFLAC
+  printf (_("FLAC, "));
+#endif
+
+#ifdef HAVE_LIBSPEEX
+  printf (_("Speex, "));
+#endif
+
+  printf (_("Ogg Vorbis.\n\n"));
+
+  printf (_("Output options\n"));
+  printf (_("  -d dev, --device dev    Use output device \"dev\". Available devices:\n"));
+  printf ("                          ");
+  printf (_("Live:"));
+
+  for(i = 0, j = 0; i < driver_count; i++) {
+    if (devices[i]->type == AO_TYPE_LIVE) {
+      printf ("%c %s", LIST_SEP(j), devices[i]->short_name);
+      j++;
+    }
+  }
+  printf ("\n                          ");
+  printf (_("File:"));
+  for(i = 0, j = 0; i < driver_count; i++) {
+    if (devices[i]->type == AO_TYPE_FILE) {
+      printf ("%c %s", LIST_SEP(j), devices[i]->short_name);
+      j++;
+    }
+  }
+  printf ("\n\n");
+
+  printf (_("  -f file, --file file    Set the output filename for a file device\n"
+	    "                          previously specified with --device.\n"));
   printf ("\n");
-  
-  printf (
-	 _("  -f, --file=filename  Set the output filename for a previously\n"
-	 "      specified file device (with -d).\n"
-	 "  -k n, --skip n  Skip the first 'n' seconds (or hh:mm:ss format)\n"
-	 "  -K n, --end n   End at 'n' seconds (or hh:mm:ss format)\n"
-	 "  -o, --device-option=k:v passes special option k with value\n"
-	 "      v to previously specified device (with -d).  See\n"
-	 "      man page for more info.\n"
-	 "  -@, --list=filename   Read playlist of files and URLs from \"filename\"\n"
-	 "  -b n, --buffer n  Use an input buffer of 'n' kilobytes\n"
-	 "  -p n, --prebuffer n  Load n%% of the input buffer before playing\n"
-	 "  -v, --verbose  Display progress and other status information\n"
-	 "  -q, --quiet    Don't display anything (no title)\n"
-	 "  -x n, --nth    Play every 'n'th block\n"
-	 "  -y n, --ntimes Repeat every played block 'n' times\n"
-	 "  -z, --shuffle  Shuffle play\n"
-	 "\n"
-	 "ogg123 will skip to the next song on SIGINT (Ctrl-C); two SIGINTs within\n"
-	 "s milliseconds make ogg123 terminate.\n"
-	 "  -l, --delay=s  Set s [milliseconds] (default 500).\n"));
+  printf (_("  --audio-buffer n        Use an output audio buffer of 'n' kilobytes\n"));
+  printf (_("  -o k:v, --device-option k:v\n"
+	    "                          Pass special option 'k' with value 'v' to the\n"
+	    "                          device previously specified with --device. See\n"
+	    "                          the ogg123 man page for available device options.\n"));
+  printf ("\n");
+
+  printf (_("Playlist options\n"));
+  printf (_("  -@ file, --list file    Read playlist of files and URLs from \"file\"\n"));
+  printf (_("  -r, --repeat            Repeat playlist indefinitely\n"));
+  printf (_("  -R, --remote            Use remote control interface\n"));
+  printf (_("  -z, --shuffle           Shuffle list of files before playing\n"));
+  printf (_("  -Z, --random            Play files randomly until interrupted\n"));
+  printf ("\n");
+
+  printf (_("Input options\n"));
+  printf (_("  -b n, --buffer n        Use an input buffer of 'n' kilobytes\n"));
+  printf (_("  -p n, --prebuffer n     Load n%% of the input buffer before playing\n"));
+  printf ("\n");
+
+  printf (_("Decode options\n"));
+  printf (_("  -k n, --skip n          Skip the first 'n' seconds (or hh:mm:ss format)\n"));
+  printf (_("  -K n, --end n           End at 'n' seconds (or hh:mm:ss format)\n"));
+  printf (_("  -x n, --nth n           Play every 'n'th block\n"));
+  printf (_("  -y n, --ntimes n        Repeat every played block 'n' times\n"));
+  printf ("\n");
+
+  printf (_("Miscellaneous options\n"));
+  printf (_("  -l s, --delay s         Set termination timeout in milliseconds. ogg123\n"
+	    "                          will skip to the next song on SIGINT (Ctrl-C),\n"
+	    "                          and will terminate if two SIGINTs are received\n"
+	    "                          within the specified timeout 's'. (default 500)\n"));
+  printf ("\n");
+  printf (_("  -h, --help              Display this help\n"));
+  printf (_("  -q, --quiet             Don't display anything (no title)\n"));
+  printf (_("  -v, --verbose           Display progress and other status information\n"));
+  printf (_("  -V, --version           Display ogg123 version\n"));
+  printf ("\n");
+
 }
