@@ -6,18 +6,22 @@
  * PLEASE READ THESE TERMS BEFORE DISTRIBUTING.                     *
  *                                                                  *
  * THE Ogg123 SOURCE CODE IS (C) COPYRIGHT 2000-2001                *
- * by Kenneth C. Arnold <ogg@arnoldnet.net> AND OTHER CONTRIBUTORS  *
+ * by Stan Seibert <volsung@xiph.org> AND OTHER CONTRIBUTORS        *
  * http://www.xiph.org/                                             *
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: compat.h,v 1.2 2001/12/19 02:52:53 volsung Exp $
+ last mod: $Id: compat.h 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
 #ifndef __COMPAT_H__
 #define __COMPAT_H__
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef __sun
 #include <alloca.h>
 #endif
