@@ -6,12 +6,12 @@
  * PLEASE READ THESE TERMS BEFORE DISTRIBUTING.                     *
  *                                                                  *
  * THE Ogg123 SOURCE CODE IS (C) COPYRIGHT 2000-2001                *
- * by Kenneth C. Arnold <ogg@arnoldnet.net> AND OTHER CONTRIBUTORS  *
+ * by Stan Seibert <volsung@xiph.org> AND OTHER CONTRIBUTORS        *
  * http://www.xiph.org/                                             *
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: ogg123.h,v 1.15 2003/09/01 23:54:02 volsung Exp $
+ last mod: $Id: ogg123.h 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
@@ -23,14 +23,16 @@
 #include "playlist.h"
 
 typedef struct ogg123_options_t {
-  long int verbosity;         /* Verbose output if > 1, quiet if 0 */
+  int verbosity;              /* Verbose output if > 1, quiet if 0 */
 
   int shuffle;                /* Should we shuffle playing? */
+  int repeat;                 /* Repeat playlist indefinitely? */
   ogg_int64_t delay;          /* delay (in millisecs) for skip to next song */
   int nth;                    /* Play every nth chunk */
   int ntimes;                 /* Play every chunk n times */
-  double seekpos;             /* Position in file to seek to */
+  double seekoff;             /* Offset to seek to */
   double endpos;              /* Position in file to play to (greater than seekpos) */
+  int seekmode;               /* DECODER_SEEK_[NONE | START | CUR */
 
   long buffer_size;           /* Size of audio buffer */
   float prebuffer;            /* Percent of buffer to fill before playing */
@@ -43,7 +45,10 @@ typedef struct ogg123_options_t {
 
   double status_freq;         /* Number of status updates per second */
 
+  int remote;                 /* Remotely controlled */
+
   playlist_t *playlist;       /* List of files to play */
+
 } ogg123_options_t;
 
 typedef struct signal_request_t {
