@@ -11,10 +11,14 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: oggvorbis_format.c,v 1.13 2003/06/24 12:34:40 giles Exp $
+ last mod: $Id: oggvorbis_format.c 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -28,6 +32,9 @@
 #include "utf8.h"
 #include "i18n.h"
 
+#ifdef HAVE_OV_READ_FILTER
+#include "vgfilter.h"
+#endif
 
 typedef struct ovf_private_t {
   OggVorbis_File vf;
@@ -38,6 +45,9 @@ typedef struct ovf_private_t {
   int bos; /* At beginning of logical bitstream */
 
   decoder_stats_t stats;
+#ifdef HAVE_OV_READ_FILTER
+  vgain_state vg;
+#endif
 } ovf_private_t;
 
 /* Forward declarations */
@@ -87,19 +97,24 @@ decoder_t* ovf_init (data_source_t *source, ogg123_options_t *ogg123_opts,
     private->stats.current_time = 0.0;
     private->stats.instant_bitrate = 0;
     private->stats.avg_bitrate = 0;
+
+#ifdef HAVE_OV_READ_FILTER
+    private->vg.scale_factor = 1.0;
+    private->vg.max_scale = 1.0;
+#endif
   } else {
-    fprintf(stderr, _("Error: Out of memory.\n"));
+    fprintf(stderr, _("ERROR: Out of memory.\n"));
     exit(1);
   }
 
   /* Initialize vorbisfile decoder */
-  
+
   ret = ov_open_callbacks (decoder, &private->vf, NULL, 0, 
 			   vorbisfile_callbacks);
 
   if (ret < 0) {
     free(private);
-/*    free(source);     nope.  caller frees. */ 
+/*    free(source);     nope.  caller frees. */
     return NULL;
   }
 
@@ -124,7 +139,40 @@ int ovf_read (decoder_t *decoder, void *ptr, int nbytes, int *eos,
     decoder->actual_fmt.rate = priv->vi->rate;
     decoder->actual_fmt.channels = priv->vi->channels;
 
-   
+    switch(decoder->actual_fmt.channels){
+    case 1:
+      decoder->actual_fmt.matrix="M";
+      break;
+    case 2:
+      decoder->actual_fmt.matrix="L,R";
+      break;
+    case 3:
+      decoder->actual_fmt.matrix="L,C,R";
+      break;
+    case 4:
+      decoder->actual_fmt.matrix="L,R,BL,BR";
+      break;
+    case 5:
+      decoder->actual_fmt.matrix="L,C,R,BL,BR";
+      break;
+    case 6:
+      decoder->actual_fmt.matrix="L,C,R,BL,BR,LFE";
+      break;
+    case 7:
+      decoder->actual_fmt.matrix="L,C,R,SL,SR,BC,LFE";
+      break;
+    case 8:
+      decoder->actual_fmt.matrix="L,C,R,SL,SR,BL,BR,LFE";
+      break;
+    default:
+      decoder->actual_fmt.matrix=NULL;
+      break;
+    }
+
+#ifdef HAVE_OV_READ_FILTER
+    vg_init(&priv->vg, priv->vc);
+#endif
+
     print_vorbis_stream_info(decoder);
     print_vorbis_comments(priv->vc, cb, decoder->callback_arg);
     priv->bos = 0;
@@ -133,12 +181,19 @@ int ovf_read (decoder_t *decoder, void *ptr, int nbytes, int *eos,
   *audio_fmt = decoder->actual_fmt;
 
   /* Attempt to read as much audio as is requested */
-  while (nbytes > 0) {
+  while (nbytes >= audio_fmt->word_size * audio_fmt->channels) {
 
     old_section = priv->current_section;
+#ifdef HAVE_OV_READ_FILTER
+    ret = ov_read_filter(&priv->vf, ptr, nbytes, audio_fmt->big_endian,
+			 audio_fmt->word_size, audio_fmt->signed_sample,
+			 &priv->current_section,
+			 vg_filter, &priv->vg);
+#else
     ret = ov_read(&priv->vf, ptr, nbytes, audio_fmt->big_endian,
 		  audio_fmt->word_size, audio_fmt->signed_sample,
 		  &priv->current_section);
+#endif
 
     if (ret == 0) {
 
@@ -147,19 +202,22 @@ int ovf_read (decoder_t *decoder, void *ptr, int nbytes, int *eos,
       break;
 
     } else if (ret == OV_HOLE) {
-      
+
       if (cb->printf_error != NULL)
 	cb->printf_error(decoder->callback_arg, INFO,
 			   _("--- Hole in the stream; probably harmless\n"));
-    
+
     } else if (ret < 0) {
-      
+
       if (cb->printf_error != NULL)
 	cb->printf_error(decoder->callback_arg, ERROR,
 			 _("=== Vorbis library reported a stream error.\n"));
-      
+
+      /* EOF */
+      *eos = 1;
+      break;
     } else {
-      
+
       bytes_read += ret;
       ptr = (void *)((unsigned char *)ptr + ret);
       nbytes -= ret;
@@ -174,7 +232,7 @@ int ovf_read (decoder_t *decoder, void *ptr, int nbytes, int *eos,
     }
 
   }
-  
+
   return bytes_read;
 }
 
@@ -298,7 +356,7 @@ void print_vorbis_stream_info (decoder_t *decoder)
 
   if (cb == NULL || cb->printf_metadata == NULL)
     return;
-    
+
   cb->printf_metadata(decoder->callback_arg, 2,
 		      _("Ogg Vorbis stream: %d channel, %ld Hz"),
 		      priv->vi->channels,
@@ -307,7 +365,7 @@ void print_vorbis_stream_info (decoder_t *decoder)
   cb->printf_metadata(decoder->callback_arg, 3,
 		      _("Vorbis format: Version %d"), 
 		      priv->vi->version);
-  
+
   cb->printf_metadata(decoder->callback_arg, 3,
 		      _("Bitrate hints: upper=%ld nominal=%ld lower=%ld "
 		      "window=%ld"), 
@@ -315,7 +373,7 @@ void print_vorbis_stream_info (decoder_t *decoder)
 		      priv->vi->bitrate_nominal,
 		      priv->vi->bitrate_lower,
 		      priv->vi->bitrate_window);
-    
+
   cb->printf_metadata(decoder->callback_arg, 3,
 		      _("Encoded by: %s"), priv->vc->vendor);
 }
@@ -326,20 +384,20 @@ void print_vorbis_comments (vorbis_comment *vc, decoder_callbacks_t *cb,
   int i;
   char *temp = NULL;
   int temp_len = 0;
-    
+
   for (i = 0; i < vc->comments; i++) {
-    
+
     /* Gotta null terminate these things */
     if (temp_len < vc->comment_lengths[i] + 1) {
       temp_len = vc->comment_lengths[i] + 1;
       temp = realloc(temp, sizeof(char) * temp_len);
     }
-    
+
     strncpy(temp, vc->user_comments[i], vc->comment_lengths[i]);
     temp[vc->comment_lengths[i]] = '\0';
-    
+
     print_vorbis_comment(temp, cb, callback_arg);
   }
-  
+
   free(temp);
 }
