@@ -11,7 +11,7 @@
  *                                                                  *
  ********************************************************************
 
- last mod: $Id: status.c,v 1.8 2002/03/19 09:50:09 msmith Exp $
+ last mod: $Id: status.c 16825 2010-01-27 04:14:08Z xiphmont $
 
  ********************************************************************/
 
@@ -324,12 +324,18 @@ void stat_format_cleanup (stat_format_t *stats)
 void status_init (int verbosity)
 {
 #if defined(HAVE_FCNTL) && defined(HAVE_UNISTD_H)
-  fcntl (STDERR_FILENO, F_SETFL, O_NONBLOCK);
+  fcntl (STDERR_FILENO, F_SETFL, fcntl(STDERR_FILENO, F_GETFL) | O_NONBLOCK);
 #endif
 
   max_verbosity = verbosity;
 }
 
+void status_deinit ()
+{
+#if defined(HAVE_FCNTL) && defined(HAVE_UNISTD_H)
+  fcntl (STDERR_FILENO, F_SETFL, fcntl(STDERR_FILENO, F_GETFL) & ~O_NONBLOCK);
+#endif
+}
 
 void status_reset_output_lock ()
 {
@@ -358,7 +364,7 @@ void status_print_statistics (stat_format_t *stats,
   /* Updating statistics is not critical.  If another thread is
      already doing output, we skip it. */
   if (pthread_mutex_trylock(&output_lock) == 0) {
-    
+
     if (decoder_statistics != NULL) {
       /* Current playback time */
       write_time_string(stats[1].arg.stringarg,
@@ -368,7 +374,7 @@ void status_print_statistics (stat_format_t *stats,
       write_time_string(stats[2].arg.stringarg,
 			decoder_statistics->total_time - 
 			decoder_statistics->current_time);
-      
+
       /* Total playback time */
       write_time_string(stats[3].arg.stringarg,
 			decoder_statistics->total_time);
@@ -383,7 +389,7 @@ void status_print_statistics (stat_format_t *stats,
 
     if (transport_statistics != NULL && 
 	transport_statistics->input_buffer_used) {
-      
+
       /* Input buffer fill % */
       stats[6].arg.doublearg = transport_statistics->input_buffer.fill;
 
@@ -394,14 +400,14 @@ void status_print_statistics (stat_format_t *stats,
 
 
     if (audio_statistics != NULL) {
-      
+
       /* Output buffer fill % */
       stats[8].arg.doublearg = audio_statistics->fill;
-      
+
       /* Output buffer state */
       write_buffer_state_string(stats[9].arg.stringarg, audio_statistics);
     }
-    
+
     last_line_len = print_statistics_line(stats);
 
     pthread_mutex_unlock(&output_lock);
